#include "Vertex.h"
#include "Camera.h"
#include "../lib/MonochromeLib.h"
#include "../lib/ECode.h"
#include <stdlib.h>

static float **stack = NULL;
static unsigned int cursor = 0;
static VertexMode currentMode = VTX_POINT;


void Vertex_Mode(VertexMode m)
{
	Vertex_Free();
	currentMode = m;
}

void Vertex_Add(float x, float y, float z)
{
	if(stack == NULL)
	{
		stack = malloc(sizeof(float*));
		cursor = 0;
	}
	else
	{
		cursor++;
		stack = realloc(stack, (cursor + 1)*sizeof(float*));
	}

	stack[cursor] = calloc(3, sizeof(float));
	stack[cursor][0] = x;
	stack[cursor][1] = y;
	stack[cursor][2] = z;
}

void Vertex_Flush(void)
{
	float view[16], proj[16], viewXproj[16], local[4][4], absolute[4][4];
	unsigned int i, j, count;
	Camera *c;

	c = Camera_Get();
	Camera_ToTransformMatrix(view);
	Camera_ToTransformMatrix(proj);

	ML_clear_vram();

	switch(currentMode)
	{
		default:
		case VTX_POINT:
			for(i = 0; i <= cursor; i++) // `i <= cursor` because cursor counts from 0
			{
				local[0][0] = stack[i][0];
				local[0][1] = stack[i][1];
				local[0][2] = stack[i][2];
				local[0][3] = 1;

				Matrix_Mult4mm(&proj, &view, &viewXproj);
				Matrix_Mult4mv(&viewXproj, &(local[0]), &(absolute[0]));

				ML_pixel(absolute[0][0], absolute[0][1], ML_BLACK);
			}
			break;
		case VTX_LINE:
			for(i = 0; i < (cursor+1) / 2; i++)
			{
				for(j = 0; j < 2; j++)
				{			
					local[j][0] = stack[i+j][0];
					local[j][1] = stack[i+j][1];
					local[j][2] = stack[i+j][2];
					local[j][3] = 1;

					Matrix_Mult4mm(&proj, &view, &viewXproj);
					Matrix_Mult4mv(&viewXproj, &(local[j]), &(absolute[j]));
				}

				ML_line(absolute[0][0], absolute[0][1], absolute[1][0], absolute[1][1], ML_BLACK);
			}
			break;
		case VTX_QUAD:
			for(i = 0; i < (cursor+1) / 4; i++)
			{
				for(j = 0; j < 4; j++)
				{			
					local[j][0] = stack[i+j][0];
					local[j][1] = stack[i+j][1];
					local[j][2] = stack[i+j][2];
					local[j][3] = 1;

					Matrix_Mult4mm(&proj, &view, &viewXproj);
					Matrix_Mult4mv(&viewXproj, &(local[j]), &(absolute[j]));
				}

				ML_line(absolute[0][0], absolute[0][1], absolute[1][0], absolute[1][1], ML_BLACK);
				ML_line(absolute[1][0], absolute[1][1], absolute[2][0], absolute[2][1], ML_BLACK);
				ML_line(absolute[2][0], absolute[2][1], absolute[3][0], absolute[3][1], ML_BLACK);
				ML_line(absolute[3][0], absolute[3][1], absolute[0][0], absolute[0][1], ML_BLACK);
			}
			break;
	}

	ML_display_vram();

	Vertex_Free();
}

void Vertex_Free(void)
{
	if(stack != NULL)
	{
		unsigned int i;
		for(i = 0; i <= cursor; i++)
		{
			free(stack[i]);
			stack[i] = NULL;
		}
		free(stack);
		stack = NULL;
	}	
}
