#include "Vector.h"
#include <math.h>

Vector3f Vector_New3f(float x, float y, float z)
{
	Vector3f v;
	v.x = x;
	v.y = y;
	v.z = z;

	return v;
}

float Vector_Dot3f(float x1, float y1, float z1, float x2, float y2, float z2)
{
	return x1 * x2 + y1 * y2 + z1 * z2;
}

float Vector_Dot3v(const Vector3f *v1, const Vector3f *v2)
{
	return v1->x * v2->x + v1->y * v2->y + v1->z * v2->z;
}

Vector3f Vector_Crossf(float x1, float y1, float z1, float x2, float y2, float z2)
{
	return Vector_New3f(
		y1 * z2 - y2 * z1,
		z1 * x2 - z2 * x1,
		x1 * y2 - x2 * y1
	);
}

Vector3f Vector_Cross3v(const Vector3f *v1, const Vector3f *v2)
{
	return Vector_New3f(
		v1->y * v2->z - v2->y * v1->z,
		v1->z * v2->x - v2->z * v1->x,
		v1->x * v2->y - v2->x * v1->y
	);
}

Vector3f Vector_Add3v(const Vector3f *v1, const Vector3f *v2)
{
	return Vector_New3f(
		v1->x + v2->x,
		v1->y + v2->y,
		v1->z + v2->z
	);
}

Vector3f Vector_Sub3v(const Vector3f *v1, const Vector3f *v2)
{
	return Vector_New3f(
		v1->x - v2->x,
		v1->y - v2->y,
		v1->z - v2->z
	);
}

float Vector_Norm3f(float x, float y, float z)
{
	return sqrt(x*x+y*y+z*z);
}

float Vector_Norm3v(const Vector3f *v)
{
	return sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
}

Vector3f Vector_Normalize3f(float x, float y, float z)
{
	float norm = Vector_Norm3f(x,y,z);
	return Vector_New3f(
		x/norm, y/norm, z/norm
	);
}

Vector3f Vector_Normalize3v(const Vector3f *v)
{
	float norm = Vector_Norm3v(v);
	return Vector_New3f(
		v->x/norm, v->y/norm, v->z/norm
	);
}